@@ -23,6 +23,7 @@
 #include <gtk/gtk.h>
 #include <string.h>
 #include <ctype.h>
+#include <locale.h>
 
 /* FIXME: Don't include htmlgdkpainter.h */
 #include "graphics/htmlgdkpainter.h"
@@ -148,11 +149,6 @@ html_box_text_finalize (GObject *self)
 		html_box_text_free_master (master);
 		g_free (master);
 		text->master = NULL;
-	} else {
-		box = HTML_BOX (self);
-
-		if (box->dom_node)
-			g_object_remove_weak_pointer (G_OBJECT (box->dom_node), (gpointer *) &(box->dom_node));
 	}
 
 	if (text->glyphs) {
@@ -349,6 +345,7 @@ html_box_text_recalc_items (HtmlBoxText *text,
 	guchar *canon_text, *canon_end;
 	PangoGlyphString *glyphs;
 	int orig_len;
+	char *ctype;
 
 	master = text->master;
 
@@ -364,8 +361,25 @@ html_box_text_recalc_items (HtmlBoxText *text,
 	/* FIXME: The context should be stored somewhere in the view tree */
 	if (context == NULL) {
 		context = gdk_pango_context_get ();
+
 		/* FIXME: This should be set from the document */
-		pango_context_set_language (context, pango_language_from_string ("fr")); 
+
+		/* modified by phill.zhang@sun.com, borrow form fontconfig */
+                ctype = setlocale (LC_CTYPE, NULL);
+                if (!ctype || !strcmp (ctype, "C"))
+                {
+                    ctype = getenv ("LC_ALL");
+                    if (!ctype)
+                    {
+                        ctype = getenv ("LC_CTYPE");
+                        if (!ctype)
+                            ctype = getenv ("LANG");
+                    }
+                }
+                if (!ctype || !strcmp(ctype, "C") || !strcmp(ctype, "POSIX"))
+		    pango_context_set_language (context, pango_language_from_string ("en")); 
+                else
+		    pango_context_set_language (context, pango_language_from_string (ctype)); 
 	}
 	
 	if (white_space == HTML_WHITE_SPACE_PRE) {
@@ -1004,15 +1018,41 @@ html_box_text_should_paint (HtmlBox *box, GdkRectangle *area, gint tx, gint ty)
 	return TRUE;
 }
 
+static AtkObject*
+html_box_text_get_accessible (HtmlBoxText *text)
+{
+	AtkObject *obj;
+
+	if (html_box_text_get_len (text) == 0)
+		return NULL;
+
+	obj = atk_gobject_accessible_for_object (G_OBJECT (text));
+	/* Accessibility is not enabled */
+	if (ATK_IS_NO_OP_OBJECT (obj))
+		return NULL;
+	return obj;
+}
+
 void 
 html_box_text_set_selection (HtmlBoxText *text, HtmlBoxTextSelection mode, gint start_index, gint end_index)
 {
+	AtkObject *obj;
+
+	if (text->selection == mode &&
+	    text->sel_start_index == start_index &&
+            text->sel_end_index == end_index)
+		return;
+ 
 	text->selection = mode;
 
 	if (start_index >= 0)
 		text->sel_start_index = start_index;
 	if (end_index >= 0)
 		text->sel_end_index = end_index;
+
+	obj = html_box_text_get_accessible (text);
+	if (obj)
+		g_signal_emit_by_name (obj, "text-selection-changed");
 }
 
 static void
