@@ -44,24 +44,23 @@ html_selection_get_text (HtmlView *view)
 		 */
 		if (text->canon_text == NULL)
 			continue;
-		ptr = (gchar *)text->canon_text;
 		switch (text->selection) {
 		case HTML_BOX_TEXT_SELECTION_NONE:
 			g_assert_not_reached ();
 			break;
 		case HTML_BOX_TEXT_SELECTION_END:
-			g_string_append_len (str, ptr, text->sel_end_index);
+			g_string_append_len (str, (gchar *)text->canon_text, g_utf8_offset_to_pointer ((gchar *)text->canon_text, text->sel_end_index) - (gchar *)text->canon_text);
 			break;
 		case HTML_BOX_TEXT_SELECTION_START:
-			g_string_append_len (str, ptr + text->sel_start_index,
-					     text->length - text->sel_start_index);
+			g_string_append_len (str, g_utf8_offset_to_pointer ((gchar *)text->canon_text, text->sel_start_index),
+					 g_utf8_offset_to_pointer ((gchar *)text->canon_text, text->length) - g_utf8_offset_to_pointer ((gchar *)text->canon_text, text->sel_start_index));
 			break;
 		case HTML_BOX_TEXT_SELECTION_FULL:
-			g_string_append_len (str, ptr, text->length);
+			g_string_append_len (str, (gchar *)text->canon_text, g_utf8_offset_to_pointer ((gchar *)text->canon_text, text->length) - (gchar *)text->canon_text);
 			break;
 		case HTML_BOX_TEXT_SELECTION_BOTH:
-			g_string_append_len (str, ptr + MIN (text->sel_start_index, text->sel_end_index),
-					 MAX (text->sel_end_index, text->sel_start_index) - MIN (text->sel_start_index, text->sel_end_index));
+			g_string_append_len (str, g_utf8_offset_to_pointer ((gchar *)text->canon_text, MIN (text->sel_start_index, text->sel_end_index)),
+					 g_utf8_offset_to_pointer ((gchar *)text->canon_text, MAX (text->sel_end_index, text->sel_start_index)) - g_utf8_offset_to_pointer ((gchar *)text->canon_text, MIN (text->sel_start_index, text->sel_end_index)));
 			break;
 		}
 	}
@@ -436,26 +435,23 @@ set_traversal (HtmlView *view, HtmlBox *root, HtmlBox *start, int *offset, int *
 		if (HTML_IS_BOX_TEXT (root)) {
 			HtmlBoxText *text = HTML_BOX_TEXT (root);
 			int text_len;
-			gchar *char_text;
 				
 			view->sel_list = g_slist_prepend (view->sel_list, text);
-			char_text = (gchar *)text->canon_text;
-			text_len = g_utf8_pointer_to_offset (char_text,
-							     char_text + text->length);
+			text_len = g_utf8_pointer_to_offset ((gchar *)text->canon_text,
+							     (gchar *)text->canon_text + text->length);
 				
 			if (*offset > 0) {
 				if (*offset < text_len) {
 					if (text_len >= *offset + *len) {
 						html_box_text_set_selection (text, 
 									     HTML_BOX_TEXT_SELECTION_BOTH, 
-									     g_utf8_offset_to_pointer (char_text, *offset) - char_text,
-									     g_utf8_offset_to_pointer (char_text, *offset + *len) - char_text);
+									     *offset, *offset + *len);
 						*len = 0;
 					}
 					else {
 						html_box_text_set_selection (text, 
 									     HTML_BOX_TEXT_SELECTION_START, 
-									     g_utf8_offset_to_pointer (char_text, *offset + *len) - char_text, -1);
+									     *offset, -1);
 						*len = *len - text_len + *offset;
 					}
 				}
@@ -470,8 +466,7 @@ set_traversal (HtmlView *view, HtmlBox *root, HtmlBox *start, int *offset, int *
 				else {
 					html_box_text_set_selection (text, 
 								     HTML_BOX_TEXT_SELECTION_END, 
-								     -1,
-								     g_utf8_offset_to_pointer (char_text, *len) - char_text);
+								     -1, *len);
 				}
 				*len = MAX (0, *len - text_len);
 			}
